VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cCoreCollection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private m_col As Collection
Private m_strFile As String

Public Enum enumCoreFields
    CoreName
    IronSection
    IronMass
    IronLength
    Hcu
    LeakCu
    AverageTurnLength
End Enum

' Initialize a new instance of this class
Private Sub Class_Initialize()
    Set m_col = New Collection
End Sub

' Add a new core of the collection
Public Sub Add(ByVal Core As cCore)
Dim i As Long

    ' Check if the core already exists
    For i = 1 To m_col.Count
        If m_col(i).Name = Core.Name Then
            Set Me.Item(i) = Core
            Exit Sub
        End If
    Next
    
    Call m_col.Add(Core)
End Sub

' Remove the specified core form the collection
Public Sub Remove(ByVal strName As String)
Dim Core As cCore
Dim i As Integer

    For i = 1 To m_col.Count
        Set Core = m_col.Item(i)
        If Core.Name = strName Then
            Call m_col.Remove(i)
            Exit Sub
        End If
    Next
End Sub

' Clear the collection
Public Sub Clear()
    Set m_col = New Collection
End Sub

' Return the item with the specified index
Public Property Get Item(ByVal index As Long) As cCore
    Set Item = m_col.Item(index)
End Property

' Replace the item with the specified index
Public Property Set Item(ByVal index As Long, ByVal Core As cCore)
    Call m_col.Remove(index)
    If index <= m_col.Count Then
        Call m_col.Add(Core, , index)
    Else
        Call m_col.Add(Core)
    End If
End Property

Public Function GetCoreByName(ByVal strName As String) As cCore
Dim Core As cCore
    
    For Each Core In m_col
        If Core.Name = strName Then
            Set GetCoreByName = Core
            Exit Function
        End If
    Next
End Function

' Return a value indicating if the specified core has changed
Public Property Get HasCoreChanged(ByVal Core As cCore) As Boolean
Dim currentCore As cCore
    
    Set currentCore = Me.GetCoreByName(Core.Name)
    If currentCore Is Nothing Then
        HasCoreChanged = True
    Else
        HasCoreChanged = Not currentCore.IsEquals(currentCore, Core)
    End If
End Property

' Return the number of core in the collection
Public Property Get Count() As Integer
    Count = m_col.Count
End Property

' Load the collection from the specified file
Public Sub LoadFromFile(ByVal strFile As String)
Dim ini As cINI
Dim Core As cCore
Dim strKeys() As String
Dim i As Integer
Dim strValue As String
Dim strValues() As String

    Set ini = New cINI
    ini.File = strFile
    
    strKeys = ini.GetKeysInSection("EI")
    For i = 0 To UBound(strKeys)
        If Len(strKeys(i)) > 0 Then
            strValue = ini.ReadString("EI", strKeys(i))
            strValues = Split(strValue, ",")
            If UBound(strValues) = 5 Then
                Set Core = New cCore
                Core.Name = strKeys(i)
                Core.IronSection = strValues(0)
                Core.IronMass = strValues(1)
                Core.IronLength = strValues(2)
                Core.Hcu = strValues(3)
                Core.Lcu = strValues(4)
                Core.AverageTurnLength = strValues(5)
                Call m_col.Add(Core)
            Else
                Call Err.Raise(10005, "Inavlid value on the file " + strFile + " key=" + strKeys(i))
            End If
        End If
    Next

    m_strFile = strFile
End Sub

Public Sub WriteToFile(Optional ByVal strFile As String)
Dim ini As cINI
Dim Core As cCore

    If Len(strFile) = 0 Then
        strFile = m_strFile
    End If
    
    Set ini = New cINI
    
    For Each Core In m_col
        Call ini.WriteString("EI", Core.Name, Core.IronSection & "," & Core.IronMass & "," & Core.IronLength & "," & Core.Hcu & "," & Core.Lcu & "," & Core.AverageTurnLength)
    Next
    
    Call ini.Save(strFile)
End Sub

' Sort the collection on the specified order and by the specified core field
Public Function Sort(ByVal eOrder As enumSortOrders, ByVal eField As enumCoreFields) As Long
    Dim lIncrement      As Long
    Dim lMainLoop       As Long
    Dim lSubLoop        As Long
    Dim Core            As cCore
    Dim lComp           As Long
    
    ' Implementation of Shell Sort algorithm using
    ' + 1 * 3 increment.
    ' Get inital Sort increment
    If (m_col.Count > 2) Then
        lIncrement = GetSuitableSortInitialIncrement(m_col.Count)
        Do Until lIncrement < 1
            For lMainLoop = lIncrement + 1 To m_col.Count
                Set Core = m_col(lMainLoop)
                ' Loop form MainLoop-Increment to 0
                For lSubLoop = (lMainLoop - lIncrement) To 1 Step -lIncrement
                    ' Retrouver l'index de la colonne  classer en cours
                    lComp = Compare(eField, m_col(lSubLoop), Core)
                    Select Case True
                    Case (lComp = 0)
                        ' No Swap
                        Exit For
                    Case (eOrder = enumSortOrders.Ascending)
                        If lComp < 1 Then
                            Exit For
                        End If
                    Case Else
                        If lComp > -1 Then
                            Exit For
                        End If
                    End Select
                    
                    Set Me.Item(lSubLoop + lIncrement) = m_col(lSubLoop)
                Next lSubLoop
                Set Me.Item(lSubLoop + lIncrement) = Core
            Next lMainLoop
            ' Get next shell sort increment value:
            lIncrement = lIncrement - 1
            lIncrement = lIncrement \ 3
        Loop

    ElseIf m_col.Count = 2 Then
        ' For only two items just check whether the second should
        ' be swapped with the first:
        Set Core = m_col(1)
        ' Retrouver l'index de la colonne  classer en cours
        lComp = Compare(eField, m_col(2), Core)
        Select Case True
        Case (lComp = 0)
            ' No Swap
        Case (eOrder = enumSortOrders.Ascending)
            If (lComp < 1) Then
                ' swap
                Call m_col.Remove(1)
                Call m_col.Add(Core)
            End If
        Case Else
            If (lComp > -1) Then
                ' swap
                Call m_col.Remove(1)
                Call m_col.Add(Core)
            End If
        End Select
    End If

    Sort = m_col.Count
    
End Function

' Part of the implementation of Shell Sort algorithm using
' + 1 * 3 increment strategy.  This function returns the
' largest increment based on +1*3 which is less than the
' sort size.
Private Function GetSuitableSortInitialIncrement(ByVal lSortSize As Long) As Long
    Dim lR      As Long
    Dim lRLast  As Long
    
    lRLast = 1
    lR = 1
    Do While lR < lSortSize
        lRLast = lR
        lR = lR * 3 + 1
    Loop
    GetSuitableSortInitialIncrement = lRLast
End Function

Private Function Compare(ByVal eField As enumCoreFields, ByVal core1 As cCore, ByVal core2 As cCore) As Long
    Select Case eField
        Case enumCoreFields.CoreName
            Compare = StrComp(core1.Name, core2.Name, vbTextCompare)
            
        Case enumCoreFields.IronSection
            Compare = core1.IronSection - core2.IronSection
            
        Case enumCoreFields.IronMass
            Compare = core1.IronMass - core2.IronMass
            
        Case enumCoreFields.IronLength
            Compare = core1.IronLength - core2.IronLength
            
        Case enumCoreFields.Hcu
            Compare = core1.Hcu - core2.Hcu
            
        Case enumCoreFields.LeakCu
            Compare = core1.Lcu - core2.Lcu
        
        Case enumCoreFields.AverageTurnLength
            Compare = core1.AverageTurnLength - core2.AverageTurnLength
            
        Case Else
            Debug.Assert False
            Compare = 0
    End Select
End Function
